"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const EncryptionService_1 = require("../../services/EncryptionService");
const locale_1 = require("../../locale");
const BaseItem_1 = require("../../models/BaseItem");
const Setting_1 = require("../../models/Setting");
const MasterKey_1 = require("../../models/MasterKey");
const registry_js_1 = require("../../registry.js");
const shim_1 = require("../../shim");
const time_1 = require("../../time");
class Shared {
    initialize(comp, props) {
        comp.state = {
            passwordChecks: {},
            stats: {
                encrypted: null,
                total: null,
            },
            passwords: Object.assign({}, props.passwords),
        };
        comp.isMounted_ = false;
        this.refreshStatsIID_ = null;
    }
    refreshStats(comp) {
        return __awaiter(this, void 0, void 0, function* () {
            const stats = yield BaseItem_1.default.encryptedItemsStats();
            comp.setState({
                stats: stats,
            });
        });
    }
    reencryptData() {
        return __awaiter(this, void 0, void 0, function* () {
            const ok = confirm(locale_1._('Please confirm that you would like to re-encrypt your complete database.'));
            if (!ok)
                return;
            yield BaseItem_1.default.forceSyncAll();
            void registry_js_1.reg.waitForSyncFinishedThenSync();
            Setting_1.default.setValue('encryption.shouldReencrypt', Setting_1.default.SHOULD_REENCRYPT_NO);
            alert(locale_1._('Your data is going to be re-encrypted and synced again.'));
        });
    }
    dontReencryptData() {
        Setting_1.default.setValue('encryption.shouldReencrypt', Setting_1.default.SHOULD_REENCRYPT_NO);
    }
    upgradeMasterKey(comp, masterKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const passwordCheck = comp.state.passwordChecks[masterKey.id];
            if (!passwordCheck) {
                alert(locale_1._('Please enter your password in the master key list below before upgrading the key.'));
                return;
            }
            try {
                const password = comp.state.passwords[masterKey.id];
                const newMasterKey = yield EncryptionService_1.default.instance().upgradeMasterKey(masterKey, password);
                yield MasterKey_1.default.save(newMasterKey);
                void registry_js_1.reg.waitForSyncFinishedThenSync();
                alert(locale_1._('The master key has been upgraded successfully!'));
            }
            catch (error) {
                alert(locale_1._('Could not upgrade master key: %s', error.message));
            }
        });
    }
    componentDidMount(comp) {
        this.componentDidUpdate(comp);
        void this.refreshStats(comp);
        if (this.refreshStatsIID_) {
            shim_1.default.clearInterval(this.refreshStatsIID_);
            this.refreshStatsIID_ = null;
        }
        this.refreshStatsIID_ = shim_1.default.setInterval(() => {
            if (!comp.isMounted_) {
                shim_1.default.clearInterval(this.refreshStatsIID_);
                this.refreshStatsIID_ = null;
                return;
            }
            void this.refreshStats(comp);
        }, 3000);
    }
    componentDidUpdate(comp, prevProps = null) {
        if (prevProps && comp.props.passwords !== prevProps.passwords) {
            comp.setState({ passwords: Object.assign({}, comp.props.passwords) });
        }
        if (!prevProps || comp.props.masterKeys !== prevProps.masterKeys || comp.props.passwords !== prevProps.passwords) {
            comp.checkPasswords();
        }
    }
    componentWillUnmount() {
        if (this.refreshStatsIID_) {
            shim_1.default.clearInterval(this.refreshStatsIID_);
            this.refreshStatsIID_ = null;
        }
    }
    checkPasswords(comp) {
        return __awaiter(this, void 0, void 0, function* () {
            const passwordChecks = Object.assign({}, comp.state.passwordChecks);
            for (let i = 0; i < comp.props.masterKeys.length; i++) {
                const mk = comp.props.masterKeys[i];
                const password = comp.state.passwords[mk.id];
                const ok = password ? yield EncryptionService_1.default.instance().checkMasterKeyPassword(mk, password) : false;
                passwordChecks[mk.id] = ok;
            }
            comp.setState({ passwordChecks: passwordChecks });
        });
    }
    decryptedStatText(comp) {
        const stats = comp.state.stats;
        const doneCount = stats.encrypted !== null ? stats.total - stats.encrypted : '-';
        const totalCount = stats.total !== null ? stats.total : '-';
        const result = locale_1._('Decrypted items: %s / %s', doneCount, totalCount);
        return result;
    }
    onSavePasswordClick(comp, mk) {
        const password = comp.state.passwords[mk.id];
        if (!password) {
            Setting_1.default.deleteObjectValue('encryption.passwordCache', mk.id);
        }
        else {
            Setting_1.default.setObjectValue('encryption.passwordCache', mk.id, password);
        }
        comp.checkPasswords();
    }
    onPasswordChange(comp, mk, password) {
        const passwords = Object.assign({}, comp.state.passwords);
        passwords[mk.id] = password;
        comp.setState({ passwords: passwords });
    }
    enableEncryptionConfirmationMessages(masterKey) {
        const msg = [locale_1._('Enabling encryption means *all* your notes and attachments are going to be re-synchronised and sent encrypted to the sync target. Do not lose the password as, for security purposes, this will be the *only* way to decrypt the data! To enable encryption, please enter your password below.')];
        if (masterKey)
            msg.push(locale_1._('Encryption will be enabled using the master key created on %s', time_1.default.unixMsToLocalDateTime(masterKey.created_time)));
        return msg;
    }
}
const shared = new Shared();
exports.default = shared;
//# sourceMappingURL=encryption-config-shared.js.map